# --------------------------------------------
# Copyright 2020, Grant Viklund
# @Author: Grant Viklund
# @Date:   2020-10-02 17:04:26
# --------------------------------------------

from rest_framework import serializers

from stylist.models import Style

class StyleSerializer(serializers.ModelSerializer):
    class Meta:
        model = Style
        fields = ("name", "enabled")



import re

from django import forms
from django.conf import settings
from django.core.exceptions import ValidationError, ImproperlyConfigured

from .models import Style
from .settings import app_settings

class StyleForm(forms.ModelForm):

    class Meta:
        model = Style
        fields = ["name", "enabled"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["name"].initial = "New Theme"
        self.fields["enabled"].initial = False


class ActiveStyleForm(forms.Form):
    active = forms.ModelChoiceField(queryset=Style.objects.all(), label="New Active Theme", empty_label="Choose Theme")

    def __init__(self, *args, **kwargs):
        if 'request' in kwargs:
            self.request = kwargs.pop("request")
        super().__init__(*args, **kwargs)      
        site = settings.SITE_ID
        if hasattr(self, 'request') and hasattr(self.request, 'site'):
            site = self.request.site.id
        self.fields["active"].queryset = Style.objects.filter(site=site)
        try:
            self.fields["active"].initial = Style.objects.filter(site=site).get(enabled=True)
        except:
            pass



class StyleEditForm(forms.ModelForm):
    class Meta:
        model = Style
        fields = ["name"]
        
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for key in self.instance.attrs:
            self.fields[key] = forms.CharField(
                required=True,
                label=settings.STYLE_SCHEMA[key]["label"],
                widget=forms.TextInput(
                    attrs={'data-type': settings.STYLE_SCHEMA[key]["type"]}
                )
            )
            self.fields[key].initial = self.instance.attrs[key]

    def clean(self):
        cleaned_data = super().clean()
        for key in list(cleaned_data):
            if key != "name":
                if settings.STYLE_SCHEMA[key]["type"] == "color":
                    self.clean_color(key)
                elif settings.STYLE_SCHEMA[key]["type"] == "number":
                    self.clean_number(key)
                elif settings.STYLE_SCHEMA[key]["type"] == "rem":
                    self.clean_rem(key)
                elif settings.STYLE_SCHEMA[key]["type"] == "px":
                    self.clean_px(key)
        
        if app_settings.USE_SASS:
            try:
                import sass
            except ModuleNotFoundError as err:
                self.add_error(None, ImproperlyConfigured("Improperly Configured: Please reinstall django-stylist with `pip install django-stylist[sass]` to add sass support"))

        return cleaned_data

    # clean key types
    def clean_color(self, key):
        data = self.cleaned_data[key]
        # match hex codes of length 3, 6, or 8
        match = re.match("^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3}|[A-Fa-f0-9]{8})$", data)
        if not match:
            self.add_error(key, ValidationError("Please enter a valid hex code"))
        return data

    def clean_number(self, key):
        data = self.cleaned_data[key]
        try:
            value = float(data)
        except:
            self.add_error(key, ValidationError("Please enter a valid number"))
        return data

    def clean_rem(self, key):
        data = self.cleaned_data[key]
        try:
            # if it is a number, add rem to end of the string
            number = float(data)
            data += "rem"
            self.cleaned_data[key] = data
        except:
            try:
                # else check if the string is a number followed by "rem"
                value = float(data[:-3])
                if data[-3:] != "rem":
                    self.add_error(key, ValidationError("Please enter a valid rem value"))
            except:
                self.add_error(key, ValidationError("Please enter a valid rem value"))
        return data

    def clean_px(self, key):
        data = self.cleaned_data[key]
        try:
            # if it is a number, add px to the end of the string
            number = float(data)
            data += "px"
            self.cleaned_data[key] = data
        except:
            try:
                # else check if the string is a number followed by "px"
                value = float(data[:-2])
                if not data[-2:] == "px":
                    self.add_error(key, ValidationError("Please enter a valid px value"))
            except:
                self.add_error(key, ValidationError("Please enter a valid px value"))
        return data

    def save(self):
        instance = super().save()
        if self.changed_data:
            for field in self.changed_data:
                if field != "name": 
                    instance.attrs[field] = self.cleaned_data[field]
            if len(self.changed_data) > 1 or self.changed_data[0] != "name":
                instance.save()
                if app_settings.USE_SASS:
                    instance.compile_attrs()
        return instance



from django.conf import settings

class AppSettings(object):

    def __init__(self, prefix):
        self.prefix = prefix

    def _setting(self, name, default):
        return getattr(settings, self.prefix + name, default)

    @property
    def USE_SASS(self):
        """ Category choices for a support request ticket """
        return self._setting('USE_SASS', False)


app_settings = AppSettings('STYLIST_')


# Generated by Django 3.1.2 on 2021-01-14 19:01

from django.db import migrations, models
import stylist.models


class Migration(migrations.Migration):

    dependencies = [
        ('stylist', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='style',
            name='attrs',
            field=models.JSONField(blank=True, default=stylist.models.default_attrs),
        ),
        migrations.AlterField(
            model_name='style',
            name='css_file',
            field=models.FileField(blank=True, null=True, upload_to=stylist.models.css_file_path, verbose_name='CSS File'),
        ),
    ]



from rest_framework.generics import CreateAPIView
from django.conf import settings
from django.contrib.sites.models import Site
from django.shortcuts import redirect

from stylist.models import Style 
from stylist.api.serializers import StyleSerializer
from stylist.settings import app_settings


class StyleCreateAPIView(CreateAPIView):
     queryset = Style.objects.all()
     serializer_class = StyleSerializer

     def perform_create(self, serializer):
          if getattr(self.request, "site", None):
               site = self.request.site
          else:
               site = Site.objects.get_current()
          instance = serializer.save(site=site)
          if app_settings.USE_SASS:
               instance.compile_attrs()

          if not Style.objects.filter(site=site, enabled=True):
               instance.enabled = True
               instance.save()

     def create(self, request, *args, **kwargs):
          response = super().create(request, *args, **kwargs)
          return redirect('stylist:stylist-index')


class StyleDuplicateAPIView(CreateAPIView):
     queryset = Style.objects.all()
     serializer_class = StyleSerializer

     def perform_create(self, serializer):
          previous = Style.objects.get(uuid=self.kwargs["uuid"])
          if getattr(self.request, "site", None):
               site = self.request.site
          else:
               site = Site.objects.get_current()
          new_name = previous.name + " copy"
          instance = serializer.save(site=site, attrs=previous.attrs, name=new_name)
          if app_settings.USE_SASS:
               instance.compile_attrs()

          if not Style.objects.filter(site=site, enabled=True):
               instance.enabled = True
               instance.save()

     def create(self, request, *args, **kwargs):
          response = super().create(request, *args, **kwargs)
          return redirect('stylist:stylist-index')



import os

from django.conf import settings
from django.contrib.staticfiles.storage import staticfiles_storage
from django.core.files.base import ContentFile
from django.core.files.storage import default_storage
from django.core.management.base import BaseCommand

from tempfile import gettempdir

from stylist.views import build_scss

class Command(BaseCommand):
    def handle(self, *args, **options):        
        data = {}
        for key, entry in settings.STYLE_SCHEMA.items():
            data[key] = entry['default']
        try:
            import sass
        except ModuleNotFoundError as err:
            raise Exception("Please reinstall django-stylist with `pip install django-stylist[sass]` to add sass support") from err
        
        custom_vars = build_scss(self, data)
        
        content = sass.compile(filename=settings.STYLIST_SCSS_TEMPLATE, include_paths=[gettempdir()])
        
        if settings.STATIC_ROOT:
            file_storage = staticfiles_storage
        else:
            file_storage = default_storage
        file_storage.delete(settings.STYLIST_DEFAULT_CSS)
        css_file = file_storage.save(settings.STYLIST_DEFAULT_CSS, ContentFile(content.encode()))

        os.remove(custom_vars.name)
        
        return 'build_scss: Wrote default css file ' + css_file



from django.urls import path

from stylist.api import views

urlpatterns = [
    path('style/create/', views.StyleCreateAPIView.as_view(), name='api-create-style'),
    path('style/<uuid:uuid>/duplicate/', views.StyleDuplicateAPIView.as_view(), name='api-duplicate-style'),
]



from django.contrib.sites.models import Site
from django.conf import settings

from .models import Style
from .settings import app_settings

def add_rgb_colors(css_attrs):
     ## adds rgb values for all colors
     for key, value in settings.STYLE_SCHEMA.items():
         if value['type'] == 'color':
             hex = css_attrs[key].lstrip("#")
             css_attrs[f"{key}-rgb"] = ",".join(tuple(str(int(hex[i:i+2], 16)) for i in (0, 2, 4)))
     return css_attrs

def get_font_families(css_attrs):
    ## adds import strings for google fonts
    font_keys = [ key for key, value in settings.STYLE_SCHEMA.items() if value['type'] == 'font' ]
    font_import = ""
    for key in font_keys:
        font_import += f"&family={css_attrs[key].replace(' ', '+')}:wght@100;200;300;400;500;600;700;800;900"
    return font_import

# ---------------
# CONTEXT PROCESSOR
# ---------------

def get_custom_styles(request):
    if getattr(request, "site", None):
        site = request.site
    else:
        site = Site.objects.get_current()
        
    try:
        style = Style.objects.filter(site=site).get(enabled=True)
        if app_settings.USE_SASS:
            custom_style = style.css_file.url
            custom_font_import = None
        else:
            css_attrs = style.attrs
            custom_style = add_rgb_colors(css_attrs)
            custom_font_import = get_font_families(css_attrs)
        
    except:
        custom_style = None
        custom_font_import = None
    try:
        if app_settings.USE_SASS:
            preview_style = request.session.get("preview_css")
            preview_font_import = None
        else:
            css_attrs = request.session.get("preview_css")
            preview_style = add_rgb_colors(css_attrs)
            preview_font_import = get_font_families(css_attrs)
    except:
        preview_style = None
        preview_font_import = None


    return {
        'custom_style': custom_style,
        'custom_font_import': custom_font_import,
        'preview_style': preview_style,
        'preview_font_import': preview_font_import
    }




# --------------------------------------------
# Copyright 2020, Grant Viklund
# @Author: Grant Viklund
# @Date:   2020-10-02 17:04:26
# --------------------------------------------

from os import path
from setuptools import setup, find_packages

from stylist.__version__ import VERSION

readme_file = path.join(path.dirname(path.abspath(__file__)), 'README.md')

try:
    from m2r import parse_from_file
    long_description = parse_from_file(readme_file)     # Convert the file to RST for PyPI
except ImportError:
    # m2r may not be installed in user environment
    with open(readme_file) as f:
        long_description = f.read()

package_metadata = {
    'name': 'django-stylist',
    'version': VERSION,
    'description': "App for updating and manipulating SASS/SCSS compiling & caching on the fly",
    'long_description': long_description,
    'url': 'https://github.com/renderbox/django-stylist/',
    'author': 'Grant Viklund',
    'author_email': 'renderbox@gmail.com',
    'license': 'MIT license',
    'classifiers': [
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    'keywords': ['django', 'app'],
}

setup(
    **package_metadata,
    packages=find_packages(),
    package_data={'stylist': ['templates/stylist/*.html']},
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=[
        'Django>=3.0,<4.1',
        'django-autoslug',
    ],
    extras_require={
        'sass': [
            'libsass'
        ],
        'dev': [                            # Packages needed by developers
            'django-crispy-forms<2.0',
            'django-allauth',
            'django-extensions',
            'djangorestframework',
        ],
        'test': ['libsass'],                         # Packages needed to run tests
        'prod': [],                         # Packages needed to run in the deployment
        'build': [                          # Packages needed to build the package
            'setuptools',
            'wheel',
            'twine',
        ],
        'docs': [                           # Packages needed to generate docs
            'coverage',
            'Sphinx',
            'recommonmark',
            'rstcheck',
            'sphinx-rtd-theme',  # Assumes a Read The Docs theme for opensource projects
        ],
    }
)



import os

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.sites.models import Site
from django.core.files.base import ContentFile
from django.core.files.storage import default_storage
from django.shortcuts import reverse, redirect
from django.urls import reverse_lazy
from django.views.generic import FormView, ListView, UpdateView, DeleteView

from datetime import datetime
from tempfile import gettempdir

from .forms import StyleForm, StyleEditForm, ActiveStyleForm
from .models import Style, css_file_path
from .settings import app_settings

def build_scss(self, data = {}):
    with open(gettempdir() + "/custom_vars.scss", "w+") as custom_vars:
        string = ""
        google_fonts = "@import url('https://fonts.googleapis.com/css2?family="
        num_fonts = 0
        for key in data:
            if key != "name":
                string += "$" + key + ": " + data[key] + ";\n"
                if settings.STYLE_SCHEMA[key]["type"] == "font":
                    if num_fonts > 0:
                        google_fonts += "&family="
                    google_fonts += data[key].replace(" ", "+")
                    google_fonts += ":wght@100;200;300;400;500;600;700;800;900"
                    num_fonts += 1
        if num_fonts > 0:
            google_fonts += "&display=swap');\n"
            string = google_fonts + string
        custom_vars.write(string)
        custom_vars.seek(0)
        
        return custom_vars

class StylistIndexView(LoginRequiredMixin, ListView):
    """
    List of styles available for editing
    """
    template_name = "stylist/index.html"
    model = Style

    def get_queryset(self):
        site_id = settings.SITE_ID
        if hasattr(self.request, 'site'):
            site_id = self.request.site.id
        return Style.objects.filter(site=site_id)

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data()
        context["form"] = StyleForm
        context["active_form"] = ActiveStyleForm
        site_id = settings.SITE_ID
        if hasattr(self.request, 'site'):
            site_id = self.request.site.id
        try:
            context["active_theme"] = Style.objects.filter(site=site_id).get(enabled=True)
        except:
            pass
        return context
    

# Stylist Preview Link - Autogenerated CSS from SASS/SCSS
class StylistPreviewView(LoginRequiredMixin, FormView):
    form_class = StyleEditForm
    template_name = "stylist/style_form.html"
    success_url = reverse_lazy("stylist:stylist-index")

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data()
        context["object"] = Style.objects.get(uuid=self.kwargs["uuid"])
        return context

    def form_valid(self, form):
        if app_settings.USE_SASS:
            import sass
            
            custom_vars = build_scss(self, form.cleaned_data)
            
            instance = Style.objects.get(uuid=self.kwargs["uuid"])
            filename = css_file_path(instance, instance.name + "_preview.css")

            # if a preview is currently active, delete the old file
            if self.request.session.get("preview_path"):
                default_storage.delete(self.request.session["preview_path"])
            content = sass.compile(filename=settings.STYLIST_SCSS_TEMPLATE, include_paths=[gettempdir()])
            preview = default_storage.save(filename, ContentFile(content.encode()))

            # bust cache to always use the most recent preview file
            timestamp = "?" + str(int(datetime.utcnow().timestamp()))
            self.request.session["preview_css"] = default_storage.url(preview) + timestamp
            self.request.session["preview_path"] = preview
            os.remove(custom_vars.name)
        else:
            self.request.session["preview_css"] = form.cleaned_data

        self.request.session["preview_uuid"] = str(self.kwargs["uuid"])
        return redirect(self.get_success_url())


# Stylist Edit Page
class StylistUpdateView(LoginRequiredMixin, UpdateView):
    model = Style
    form_class = StyleEditForm

    def get_object(self, *args, **kwargs):
        return Style.objects.get(uuid=self.kwargs.get("uuid"))


# Change which Style is currently active for the site
class StylistActiveView(LoginRequiredMixin, FormView):
    form_class = ActiveStyleForm
    template_name = "stylist/active_form.html"

    def form_valid(self, form):
        instance = form.cleaned_data["active"]
        instance.enabled = True
        instance.save()
        return redirect('stylist:stylist-index')

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data()
        site_id = settings.SITE_ID
        if hasattr(self.request, 'site'):
            site_id = self.request.site.id
        try:
            context["active_theme"] = Style.objects.filter(site=site_id).get(enabled=True)
        except:
            pass
        return context

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['request'] = self.request
        return kwargs


# Stylist Detail Page


class StylistDeleteView(LoginRequiredMixin, DeleteView):
    model = Style
    template_name = 'stylist/style_delete.html'

    def get_object(self, queryset=None):
        return Style.objects.get(uuid=self.kwargs.get("uuid"))

    def get_success_url(self):
        return reverse('stylist:stylist-index')


# Ends preview mode
@login_required
def end_preview(request):
    preview_path = request.session.pop("preview_path", None)
    request.session.pop("preview_css", None)
    request.session.pop("preview_uuid", None)
    if preview_path:
        default_storage.delete(preview_path)
    return redirect(request.META.get('HTTP_REFERER'))
